Lighttpd patch to enable the "Content-Type" extended attribute on FreeBSD.

Not yet integrated with lighttpd build system. To build using ports,

# cd /usr/ports/www/lighttpd
# make configure
-output omitted-
# cd work/lighttpd-1.4.*
# patch -p1 < path/to/this.diff
-output omitted-
# sed -i.orig 's/DEFS =/DEFS = -DHAVE_XATTR/' src/Makefile
# cd ../..
# make install
-output omitted-

After installation, when lighttpd is running with the "xattrs" option enabled
in the config file, use this to change a file's Content-Type:

# setextattr user Content-Type text/plain path/to/www/file

This will make the file be sent using the corresponding Content-Type header as
well as make it appear as the content type in question in mod_dirlisting.

For more information, see setextattr(8), extattr(9) and related manpages.

This code should also work on NetBSD and OpenBSD, as they seem to share the
extattr API with FreeBSD. However, this was not tested.

Author: Moritz Wilhelmy <mw barfooze de>, 2014

--- lighttpd-1.4.35/src/mod_dirlisting.c.orig	2014-03-06 15:08:00.000000000 +0100
+++ lighttpd-1.4.35/src/mod_dirlisting.c	2014-05-21 22:23:22.000000000 +0200
@@ -18,6 +18,13 @@
 #include <unistd.h>
 #include <time.h>
 
+#if defined(HAVE_XATTR) && defined(__FreeBSD__)
+
+#include <sys/types.h>
+#include <sys/extattr.h>
+
+#endif
+
 /**
  * this is a dirlisting for a lighttpd plugin
  */
@@ -822,6 +829,7 @@
 		content_type = NULL;
 #ifdef HAVE_XATTR
 
+#ifndef __FreeBSD__
 		if (con->conf.use_xattr) {
 			memcpy(path_file, DIRLIST_ENT_NAME(tmp), tmp->namelen + 1);
 			attrlen = sizeof(attrval) - 1;
@@ -830,6 +838,18 @@
 				content_type = attrval;
 			}
 		}
+#else
+		if (con->conf.use_xattr) {
+			memcpy(path_file, DIRLIST_ENT_NAME(tmp), tmp->namelen + 1);
+			attrlen = sizeof(attrval)-1;
+			if(-1 != (attrlen = extattr_get_file(path, EXTATTR_NAMESPACE_USER,
+				"Content-Type", attrval, attrlen))) {
+				attrval[attrlen] = '\0';
+				content_type = attrval;
+			}
+		}
+
+#endif
 #endif
 
 		if (content_type == NULL) {
--- lighttpd-1.4.35/src/stat_cache.c.orig	2014-03-06 15:08:00.000000000 +0100
+++ lighttpd-1.4.35/src/stat_cache.c	2014-05-21 22:23:51.000000000 +0200
@@ -211,6 +211,8 @@
 }
 
 #ifdef HAVE_XATTR
+
+#ifndef __FreeBSD__
 static int stat_cache_attr_get(buffer *buf, char *name) {
 	int attrlen;
 	int ret;
@@ -224,6 +226,26 @@
 	}
 	return ret;
 }
+#else
+#include <sys/types.h>
+#include <sys/extattr.h>
+#include <stdio.h>
+
+static int stat_cache_attr_get(buffer *buf, char *name) {
+	ssize_t attrlen = 1024;
+
+	buffer_prepare_copy(buf, attrlen);
+
+	if(-1 != (attrlen = extattr_get_file(name, EXTATTR_NAMESPACE_USER,
+		"Content-Type", buf->ptr, attrlen-1))) {
+
+		buf->used = attrlen + 1;
+		buf->ptr[attrlen] = '\0';
+	}
+	return attrlen != -1;
+}
+#endif
+
 #endif
 
 /* the famous DJB hash function for strings */
